# Generated by roxygen2: do not edit by hand

S3method(plot,EnsembleSample)
export(EnsembleData)
export(EnsembleFit)
export(EnsemblePrior)
export(EnsembleSample)
export(IndLTPrior)
export(IndSTPrior)
export(KalmanFilter_back)
export(ShaSTPrior)
export(TruthPrior)
export(fit_ensemble_model)
export(gen_sample)
export(generate_sample)
export(get_mcmc_ensemble_model)
export(get_mle)
export(get_parameters)
export(get_transformed_data)
export(get_transformed_data_dri)
export(prior_ensemble_model)
export(sample_prior)
exportClasses(EnsembleData)
exportClasses(EnsembleFit)
exportClasses(EnsemblePrior)
exportClasses(EnsembleSample)
exportClasses(IndLTPrior)
exportClasses(IndSTPrior)
exportClasses(ShaSTPrior)
exportClasses(TruthPrior)
import(Rcpp)
import(cowplot)
import(dplyr)
import(ggplot2)
import(matrixcalc)
import(methods)
import(reshape2)
import(tibble)
importClassesFrom(rstan,stanfit)
importFrom(cowplot,plot_grid)
importFrom(rstan,optimizing)
importFrom(rstan,sampling)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,rnorm)
useDynLib(EcoEnsemble, .registration = TRUE)
